/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 05.10.2009 - [JR] - creation
 */
package com.sibvisions.apps.packung;

import javax.rad.model.SortDefinition;
import javax.rad.model.condition.Equals;
import javax.rad.remote.IConnectionConstants;
import javax.rad.server.IConfiguration;
import javax.rad.server.ISession;
import javax.rad.server.SessionContext;
import javax.rad.type.bean.IBean;

import com.sibvisions.apps.packung.business.UserRegistration;
import com.sibvisions.apps.server.object.DBWorkScreenAccess;
import com.sibvisions.apps.server.object.IWorkScreenAccess;
import com.sibvisions.rad.persist.jdbc.DBAccess;
import com.sibvisions.rad.persist.jdbc.DBStorage;
import com.sibvisions.rad.server.GenericBean;
import com.sibvisions.rad.server.config.DBObjects;

/**
 * The <code>Session</code> class is the lifecycle object for the MasterSession.
 * 
 * @author Ren Jahn
 */
public class Session extends GenericBean
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Inits the Session.
	 */
	public Session()
	{
		DBStorage.setDefaultAutoLinkReference(false);
		DBStorage.setDefaultAllowedValues(false);
		DBStorage.setDefaultDefaultValue(false);
	}
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Creates a new database access object.
	 * 
	 * @return the new database access object
	 * @throws Exception if the connection can not be opened
	 */
	public DBAccess getNewDBAccess() throws Exception
	{
		IConfiguration cfgSession = SessionContext.getCurrentSessionConfig();
	
		DBAccess dba = DBAccess.getDBAccess(cfgSession.getProperty("/application/securitymanager/database/url"));

		dba.setUsername(cfgSession.getProperty("/application/securitymanager/database/username"));
		dba.setPassword(cfgSession.getProperty("/application/securitymanager/database/password"));
		dba.open();
				
		put("dBAccess", dba);
		
		DBStorage dbsUser = new DBStorage();
		dbsUser.setDBAccess(dba);
		dbsUser.setFromClause("USERS");
		dbsUser.open();
		
		ISession session = SessionContext.getCurrentSession();
		
		IBean bnUser = dbsUser.fetchBean(new Equals("USERNAME", session.getUserName()));
		
		Object userId = bnUser.get(DBObjects.getColumnName(session.getConfig(), "USERS", "ID"));
		
		session.setProperty(IConnectionConstants.PREFIX_SERVER + IConnectionConstants.PREFIX_SESSION + "ID", userId);
		
		put("userId", userId);
		
		//let the client know that the user is an automatic created user
		if (DBObjects.getYesValue(session.getConfig()).equals(bnUser.get(DBObjects.getColumnName(session.getConfig(), "USERS", "REGISTERED"))))
		{
			session.setProperty(IConnectionConstants.PREFIX_SERVER + IConnectionConstants.PREFIX_SESSION + "REGISTERED", "Y");
		}
		
		return dba;
	}

	/**
	 * Returns the access to the database.
	 * 
	 * @return the access to the database
	 * @throws Exception if the datasource can not be opened
	 */
	public DBAccess getDBAccess() throws Exception
	{
		DBAccess dba = (DBAccess)get("dBAccess");
		
		if (dba == null)
		{
			dba = getNewDBAccess();
					
			put("dBAccess", dba);
		}
		
		return dba;
	}
	
	/**
	 * Gets the ID of the current logged on user.
	 * 
	 * @return the ID of the current logged on user.
	 */
	public Object getUserId()
	{
		return get("userId");
	}
	
	/**
	 * Returns the workscreen access object. With this object we can retrieve the user based
	 * application configuration.
	 * 
	 * @return the workscreen access object
	 * @throws Exception if it's not possible to access the database
	 */
	public IWorkScreenAccess getWorkScreenAccess() throws Exception
	{
		DBWorkScreenAccess workScreenAccess = (DBWorkScreenAccess)get("workScreenAccess");
		if (workScreenAccess == null)
		{
			workScreenAccess = new DBWorkScreenAccess();
			workScreenAccess.setDBDataSource(getDBAccess());

			put("workScreenAccess", workScreenAccess);
		}
		return workScreenAccess;
	}
	
	/**
	 * Returns the config DBStorage.
	 * 
	 * @return the config DBStorage.
	 * @throws Exception if the teams DBStorage couldn't be initialized.
	 */
	public DBStorage getConfig() throws Exception
	{
		DBStorage dbsConfig = (DBStorage)get("config");
		
		if (dbsConfig == null)
		{
			dbsConfig = new DBStorage();
			dbsConfig.setDBAccess(getDBAccess());
			dbsConfig.setFromClause("V_COFG_CONFIG");
			dbsConfig.setWritebackTable("CONFIG");
			dbsConfig.setDefaultSort(new SortDefinition("SORT"));
			dbsConfig.open();
			
			put("config", dbsConfig);
		}
		
		return dbsConfig;
	}

	/**
	 * Returns the language information DBStorage.
	 * 
	 * @return the language information DBStorage
	 * @throws Exception if the language information DBStorage couldn't be initialized
	 */
	public DBStorage getLanguageInfos() throws Exception
	{
		DBStorage dbsLangInfo = (DBStorage)get("languageInfos");
		
		if (dbsLangInfo == null)
		{
			dbsLangInfo = new DBStorage();
			dbsLangInfo.setDBAccess(getDBAccess());
			dbsLangInfo.setFromClause("V_GLOBAL_LANGUAGEINFOS");
			dbsLangInfo.setDefaultSort(new SortDefinition("LANGUAGE"));
			dbsLangInfo.open();
			
			put("languageInfos", dbsLangInfo);
		}
		
		return dbsLangInfo;
	}
	
	/**
	 * Gets the user registration business object.
	 * 
	 * @return the business object for user registrations
	 */
	public UserRegistration getRegistration()
	{
		UserRegistration ureg = (UserRegistration)get("registration");
		
		if (ureg == null)
		{
			ureg = new UserRegistration();
			
			put("registration", ureg);
		}
		
		return ureg;
	}

	/**
	 * Returns the user defaults DBStorage.
	 * 
	 * @return the user defaults DBStorage.
	 * @throws Exception if the user defaults DBStorage couldn't be initialized.
	 */
	public DBStorage getUserDefaults() throws Exception
	{
		DBStorage userDefaults = (DBStorage)get("userDefaults");
		if (userDefaults == null)
		{
			userDefaults = new DBStorage();
			userDefaults.setDBAccess(getDBAccess());
			userDefaults.setFromClause("V_GLOBAL_USER_DEFAULTS");
			userDefaults.setWritebackTable("USERS");
			userDefaults.setRestrictCondition(new Equals("ID", getUserId()));
			userDefaults.open();
			
			put("userDefaults", userDefaults);
		}
		return userDefaults;
	}

	/**
	 * Returns the competitions lookup DBStorage.
	 * 
	 * @return the competitions lookup DBStorage.
	 * @throws Exception if the competitions lookup DBStorage couldn't be initialized.
	 */
	public DBStorage getCompetitionsLook() throws Exception
	{
		DBStorage competitionsLook = (DBStorage)get("competitionsLook");
		if (competitionsLook == null)
		{
			competitionsLook = new DBStorage();
			competitionsLook.setDBAccess(getDBAccess());
			competitionsLook.setFromClause("V_GLOBAL_COMPETITIONS_LOOK");
			competitionsLook.setDefaultSort(new SortDefinition(false, "ID"));
			competitionsLook.open();
			
			put("competitionsLook", competitionsLook);
		}
		return competitionsLook;
	}
	
	/**
	 * Returns the languagesLook DBStorage.
	 * 
	 * @return the languagesLook DBStorage.
	 * @throws Exception if the languagesLook DBStorage couldn't be initialized.
	 */
	public DBStorage getLanguagesLook() throws Exception
	{
		DBStorage languagesLook = (DBStorage)get("languagesLook");
		if (languagesLook == null)
		{
			languagesLook = new DBStorage();
			languagesLook.setDBAccess(getDBAccess());
			languagesLook.setFromClause("V_GLOBAL_LANGUAGES_LOOK");
			languagesLook.open();
			
			put("languagesLook", languagesLook);
		}
		return languagesLook;
	}
	
}	// Session
