/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 13.10.2009 - [JR] - creation
 * 26.10.2009 - [JR] - sendPrivateGroup: reuse existing group code
 *                   - joinPrivateGroup implemented
 * 27.10.2009 - [JR] - create: rollback/commit handling changed  
 * 28.10.2009 - [JR] - joinPrivateGroup: compare Number
 * 06.03.2010 - [JR] - #71: changed Bean handling                 
 */
package com.sibvisions.apps.packung.business;

import java.math.BigDecimal;
import java.security.SecureRandom;
import java.util.Hashtable;
import java.util.List;
import java.util.UUID;

import javax.mail.internet.InternetAddress;
import javax.rad.application.ILauncher;
import javax.rad.model.condition.Equals;
import javax.rad.remote.IConnectionConstants;
import javax.rad.server.SessionContext;
import javax.rad.type.bean.Bean;
import javax.rad.type.bean.IBean;

import com.sibvisions.rad.persist.jdbc.DBAccess;
import com.sibvisions.rad.persist.jdbc.DBStorage;
import com.sibvisions.util.Mail;
import com.sibvisions.util.type.BeanUtil;
import com.sibvisions.util.type.CommonUtil;

/**
 * The <code>UserRegistration</code> class is the business object for registering
 * new users.
 * 
 * @author Ren Jahn
 */
public class UserRegistration
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** Contains allowed characters for new passwords. */
	private static final String CHARS_PWD 		= "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789abcdefghijklmnopqrstuvwxyz._$!?";

	/** Contains allowed characters for bet group code. */
	private static final String CHARS_BETGROUP 	= "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
	
	
	/** the last created activation key. */
	private String sLastActivationKey;
	
	/** the last created random code. */
	private String sLastCode;

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Creates a new user.
	 * 
	 * @param pFirstName the first name
	 * @param pLastName the last name
	 * @param pUserName the username
	 * @param pEmail the email of the user
	 * @param pPassword the password
	 * @param pPasswordConfirm the confirm password
	 * @param pLangId the id of the preferred language
	 * @return a hashtable with fieldnames as key and errormessages as value
	 * @throws RuntimeException if the database is not available
	 */
	public Hashtable<String, String> create(String pFirstName,
											String pLastName,
											String pUserName, 
			               					String pEmail, 
			               					String pPassword, 
			               					String pPasswordConfirm, 
			               					BigDecimal pLangId)
	{
		Hashtable<String, String> htResult = new Hashtable<String, String>();
		
		//PASSWORD(s) check
		
		if (isEmpty(pPassword))
		{
			htResult.put("PWD", "Empty password is not allowed!");
		}
		
		if (!pPasswordConfirm.equals(pPassword))
		{
			htResult.put("PWD_CONFIRM", "Password and confirmation password are different!");
		}
		
		if (isEmpty(pEmail))
		{
			htResult.put("EMAIL", "Empty email adress is not allowed!");
		}
		else
		{
			//EMAIL check
			
			try
			{
				new InternetAddress(pEmail);
			}
			catch (Exception e)
			{
				htResult.put("EMAIL", "Email adress is not valid!");
			}
			
			int iPos = pEmail.indexOf('@');
			
			if (iPos <= 0 || iPos == pEmail.length() - 1)
			{
				htResult.put("EMAIL", "Email adress is not valid!");
			}
		}
		
		//USERNAME check
		
		if (isEmpty(pUserName))
		{
			htResult.put("USER", "Empty username is not allowed!");
		}
		else
		{
			DBAccess dba = null;
			
			try
			{
				dba = (DBAccess)SessionContext.getCurrentSession().get("newDBAccess");
				
				dba.getConnection().setAutoCommit(false);
				
				//for security reasons not defined in the Session LCO
				DBStorage dbsUser = new DBStorage();
				dbsUser.setDBAccess(dba);
				dbsUser.setWritebackTable("USERS");
				dbsUser.open();
				
				//CHECK if the user exists
				IBean bnUser = dbsUser.fetchBean(new Equals("USERNAME", pUserName));
	
				if (bnUser != null)
				{
					htResult.put("USER", "User already exists!");
				}
				
				//Create a new user, when no errors occured!
				if (htResult.size() == 0)
				{
					//----------------------------------------------------------------------
					// user creation
					//----------------------------------------------------------------------
					
					DBStorage dbsInfoText = new DBStorage();
					dbsInfoText.setDBAccess(dba);
					dbsInfoText.setFromClause("V_GLOBAL_LANGUAGEINFOS");
					dbsInfoText.open();
					
					IBean bnText;
					
					if (pLangId == null)
					{
						bnText = dbsInfoText.fetchBean(new Equals("STANDARD", "Y"));
					}
					else
					{
						bnText = dbsInfoText.fetchBean(new Equals("ID", pLangId));
					}
					
					if (bnText == null)
					{
						throw new SecurityException("Language settings not found!");
					}
					
					String sActivationKey = UUID.randomUUID().toString();
					
					sLastActivationKey = sActivationKey;
					
					Bean bean = new Bean();
					
					bean.put("FIRSTNAME", pFirstName);
					bean.put("LASTNAME", pLastName);
					bean.put("USERNAME", pUserName);
					bean.put("PASSWORD", pPassword);
					bean.put("EMAIL", pEmail);
					bean.put("ACTIVE", "N");
					bean.put("ACTIVATIONKEY", sActivationKey);
					bean.put("LANG_ID", bnText.get("ID"));
					
					dbsUser.insertBean(bean);
				
					Hashtable<Object, Object> htConfig = getConfig();
					
					try
					{
						//----------------------------------------------------------------------
						// Prepare and send activation email
						//----------------------------------------------------------------------

						String sEmailText = (String)bnText.get("REGEMAIL"); 
						
						if (sEmailText == null)
						{
							sEmailText = "[ACTIVATIONURL]\n\n[USERNAME]\n[PASSWORD]";
						}
						
						//update placeholders!
						sEmailText = sEmailText.replace("[FIRSTNAME]", CommonUtil.nvl(pFirstName, ""));
						sEmailText = sEmailText.replace("[LASTNAME]", CommonUtil.nvl(pLastName, ""));
						sEmailText = sEmailText.replace("[USERNAME]", pUserName);
						sEmailText = sEmailText.replace("[PASSWORD]", pPassword);
						
						String sUrl = (String)SessionContext.getCurrentSession().getProperty(IConnectionConstants.PREFIX_CLIENT + ILauncher.PARAM_SERVERBASE);
						
						if (sUrl != null)
						{
							sUrl += "?activationkey=" + sActivationKey;

							sEmailText = sEmailText.replace("[ACTIVATIONURL]", sUrl);
						}
						else
						{
							sEmailText = sEmailText.replace("[ACTIVATIONURL]", "");
						}
						
						sEmailText = sEmailText.replace("[ACTIVATIONKEY]", sActivationKey);
						
						Mail mail = new Mail((String)htConfig.get("Mail.host"), 
								             (String)htConfig.get("Mail.port"), 
								             (String)htConfig.get("Mail.username"), 
								             (String)htConfig.get("Mail.password"));
						
						mail.send((String)htConfig.get("Mail.from"), 
								  pEmail, 
								  null, 
								  (String)htConfig.get("Mail.subject.registration"), 
								  sEmailText);
						
						dba.commit();
					}
					catch (Exception e)
					{
						throw new RuntimeException("Sending activation mail failed!", e);
					}
				}
			}
			catch (Throwable th)
			{
				if (dba != null)
				{
					try
					{
						dba.rollback();
					}
					catch (Exception e)
					{
						//nothing to be done
					}
				}
				
				throw new RuntimeException("Registration failed!", th);
			}
		}
		
		return htResult;
	}
	
	/**
	 * Checks if a string contains characters.
	 * 
	 * @param pValue a string
	 * @return <code>true</code> if <code>pValue</code> contains characters
	 */
	private boolean isEmpty(String pValue)
	{
		return pValue == null || pValue.trim().length() == 0;
	}

	/**
	 * Removes/Unregisters a user and all its content.
	 * 
	 * @param pUserName the user name
	 * @throws Exception if the user can not be deleted
	 */
	public void delete(String pUserName) throws Exception
	{
		try
		{
			DBAccess dba = (DBAccess)SessionContext.getCurrentSession().get("newDBAccess");
			
			DBStorage dbsUser = new DBStorage();
			dbsUser.setDBAccess(dba);
			dbsUser.setWritebackTable("USERS");
			dbsUser.open();
			
			List<IBean> liBeans = dbsUser.fetchBean(new Equals("USERNAME", pUserName), null, 0, -1);
			
			//the username is unique and it's not possible that a user exists more than once!
			if (liBeans.size() == 1)
			{
				//delete the user (db will cascade all user-data)
				dbsUser.deleteBean(liBeans.get(0));
			}
			else
			{
				throw new SecurityException("User '" + pUserName + "' was not found!");
			}
		}
		catch (Throwable th)
		{
			throw new SecurityException("It's not possible to delete the user!", th);
		}
	}
	
	/**
	 * Sends a new password to a specific user.
	 * 
	 * @param pUserName the username
	 * @param pClientLanguage the current client language
	 * @throws Exception if the user is unknown, the email address is missing or the mail server is not
	 *                   configured
	 */
	public void setNewPassword(String pUserName, String pClientLanguage) throws Exception
	{
		DBAccess dba = null;
		
		try
		{
			dba = (DBAccess)SessionContext.getCurrentSession().get("newDBAccess");
			
			dba.getConnection().setAutoCommit(false);
	
			//----------------------------------------------------------------------
			// Search the user
			//----------------------------------------------------------------------
	
			//for security reasons not defined in the Session LCO
			DBStorage dbsUser = new DBStorage();
			dbsUser.setDBAccess(dba);
			dbsUser.setWritebackTable("USERS");
			dbsUser.open();
			
			//CHECK if the user exists
			IBean bnUser = dbsUser.fetchBean(new Equals("USERNAME", pUserName));
			
			if (bnUser == null)
			{
				throw new SecurityException("Unknown user '" + pUserName + "'");
			}
			
			//----------------------------------------------------------------------
			// Prepare mail
			//----------------------------------------------------------------------
	
			String sTo = (String)bnUser.get("EMAIL");
			
			if (sTo == null)
			{
				throw new IllegalArgumentException("Unknown email address!");
			}
	
			DBStorage dbsInfoText = new DBStorage();
			dbsInfoText.setDBAccess(dba);
			dbsInfoText.setFromClause("V_GLOBAL_LANGUAGEINFOS");
			dbsInfoText.open();
	
			IBean bnText = null;
			
			if (bnUser.get("LANG_ID") == null)
			{
				//try the client language setting
				if (pClientLanguage != null)
				{
					bnText = dbsInfoText.fetchBean(new Equals("CODE", pClientLanguage));
				}
				
				if (bnText == null)
				{
					bnText = dbsInfoText.fetchBean(new Equals("STANDARD", "Y"));
				}
			}
			else
			{
				bnText = dbsInfoText.fetchBean(new Equals("ID", bnUser.get("LANG_ID")));
			}
			
			if (bnText == null)
			{
				throw new SecurityException("Language settings not found!");
			}
			
			Hashtable<Object, Object> htConfig = getConfig();
	
			String sText = (String)bnText.get("PWDEMAIL");
	
			String sNewPassword = createCode(CHARS_PWD);
			
			if (sText == null)
			{
				sText = "[USERNAME]\n[PASSWORD]";
			}
			
			//update placeholders!
			sText = sText.replace("[FIRSTNAME]", CommonUtil.nvl((String)bnUser.get("FIRSTNAME"), ""));
			sText = sText.replace("[LASTNAME]", CommonUtil.nvl((String)bnUser.get("LASTNAME"), ""));
			sText = sText.replace("[USERNAME]", (String)bnUser.get("USERNAME"));
			sText = sText.replace("[PASSWORD]", sNewPassword);
			
			//----------------------------------------------------------------------
			// Change user
			//----------------------------------------------------------------------
	
			Bean bnNewUser = new Bean();
			bnNewUser.put("ID", bnUser.get("ID"));
			bnNewUser.put("PASSWORD", sNewPassword);
			
			dbsUser.updateBean(bnNewUser);
			
			//----------------------------------------------------------------------
			// Send mail
			//----------------------------------------------------------------------

			try
			{
				Mail mail = new Mail((String)htConfig.get("Mail.host"), 
						             (String)htConfig.get("Mail.port"), 
						             (String)htConfig.get("Mail.username"), 
						             (String)htConfig.get("Mail.password"));
		
				mail.send((String)htConfig.get("Mail.from"), 
						  sTo, 
						  null, 
						  (String)htConfig.get("Mail.subject.lostpassword"), 
						  sText);
				
				//save the new password after the email was sent successful!
				dba.commit();
			}
			catch (Exception e)
			{
				throw new RuntimeException("Sending password mail failed!", e);
			}
		}
		catch (Throwable th)
		{
			if (dba != null)
			{
				try
				{
					dba.rollback();
				}
				catch (Exception e)
				{
					//nothing to be done
				}
			}
			
			throw new RuntimeException("Change password failed!", th);
		}
	}

	/**
	 * Creates a bet group code and sends this code to specific email addresses. This
	 * will be used to invite users to private bet groups.
	 * 
	 * @param pBetGroupId the bet group id
	 * @param pClientLanguage the language code from the client
	 * @param pRecipients a list of email recipients
	 */
	public void sendPrivateGroup(BigDecimal pBetGroupId, String pClientLanguage, String pRecipients)
	{
		DBAccess dba = null;
		
		try
		{
			dba = (DBAccess)SessionContext.getCurrentSession().get("newDBAccess");
			
			dba.getConnection().setAutoCommit(false);
	
			//----------------------------------------------------------------------
			// Search the bet group
			//----------------------------------------------------------------------
	
			//for security reasons not defined in the Session LCO
			DBStorage dbsBetGroups = new DBStorage();
			dbsBetGroups.setDBAccess(dba);
			dbsBetGroups.setWritebackTable("BETGROUPS");
			dbsBetGroups.open();
			
			//CHECK if the group exists
			IBean bnGroup = dbsBetGroups.fetchBean(new Equals("ID", pBetGroupId));
			
			if (bnGroup == null)
			{
				throw new SecurityException("Bet group not found!");
			}
			
			//----------------------------------------------------------------------
			// Prepare mail
			//----------------------------------------------------------------------
	
			if (pRecipients == null)
			{
				throw new IllegalArgumentException("Unknown email address!");
			}
	
			DBStorage dbsInfoText = new DBStorage();
			dbsInfoText.setDBAccess(dba);
			dbsInfoText.setFromClause("V_GLOBAL_LANGUAGEINFOS");
			dbsInfoText.open();
	
			IBean bnText = null;
			
			//try the client language setting
			if (pClientLanguage != null)
			{
				bnText = dbsInfoText.fetchBean(new Equals("CODE", pClientLanguage));
			}
			
			if (bnText == null)
			{
				bnText = dbsInfoText.fetchBean(new Equals("STANDARD", "Y"));
			}
			
			if (bnText == null)
			{
				throw new SecurityException("Language settings not found!");
			}
			
			Hashtable<Object, Object> htConfig = getConfig();
	
			String sText = (String)bnText.get("GROUPEMAIL");
	
			String sCode = (String)bnGroup.get("CODE");
			
			if (sCode == null)
			{
				//add the id to ensure uniqueness
				sCode = bnGroup.get("ID") + createCode(CHARS_BETGROUP);
				
				//----------------------------------------------------------------------
				// Change group
				//----------------------------------------------------------------------
		
				Bean bnNewGroup = new Bean();
				bnNewGroup.put("ID", bnGroup.get("ID"));
				bnNewGroup.put("CODE", sCode);
				
				dbsBetGroups.updateBean(bnNewGroup);
			}
			else
			{
				sLastCode = sCode;
			}
			
			if (sText == null)
			{
				sText = "[GROUPNAME]\n[CODE]";
			}
			
			//update placeholders!
			sText = sText.replace("[GROUPNAME]", (String)bnGroup.get("BETGROUP"));
			sText = sText.replace("[CODE]", sCode);
			
			//----------------------------------------------------------------------
			// Send mail
			//----------------------------------------------------------------------

			try
			{
				String sUrl = (String)SessionContext.getCurrentSession().getProperty(IConnectionConstants.PREFIX_CLIENT + ILauncher.PARAM_SERVERBASE);
				
				if (sUrl != null)
				{
					sText = sText.replace("[GROUPURL]", sUrl);
				}
				else
				{
					sText = sText.replace("[GROUPURL]", "");
				}
				
				Mail mail = new Mail((String)htConfig.get("Mail.host"), 
						             (String)htConfig.get("Mail.port"), 
						             (String)htConfig.get("Mail.username"), 
						             (String)htConfig.get("Mail.password"));
		
				mail.send((String)htConfig.get("Mail.from"), 
						  pRecipients, 
						  null, 
						  (String)htConfig.get("Mail.subject.betgroup"), 
						  sText);
				
				//save the new code after the email was sent successful!
				dba.commit();
			}
			catch (Exception e)
			{
				throw new RuntimeException("Sending bet group mail failed!", e);
			}
		}
		catch (Throwable th)
		{
			if (dba != null)
			{
				try
				{
					dba.rollback();
				}
				catch (Exception e)
				{
					//nothing to be done
				}
			}
			
			throw new RuntimeException("Generating bet group code failed!", th);
		}
	}
	
	/**
	 * Add a user to a bet group. The group code will be needed to add a user to a group.
	 *  
	 * @param pUserId the id of the user to join
	 * @param pGroupCode a group code
	 * @return the group name or <code>null</code> if there is no group with the specified code
	 */
	public String joinPrivateGroup(BigDecimal pUserId, String pGroupCode)
	{
		DBAccess dba = null;
		
		try
		{
			dba = (DBAccess)SessionContext.getCurrentSession().get("newDBAccess");
			
			dba.getConnection().setAutoCommit(false);
			
			//----------------------------------------------------------------------
			// Search the bet group
			//----------------------------------------------------------------------
	
			//for security reasons not defined in the Session LCO
			DBStorage dbsBetGroups = new DBStorage();
			dbsBetGroups.setDBAccess(dba);
			dbsBetGroups.setFromClause("BETGROUPS");
			dbsBetGroups.open();
			
			//CHECK if the group exists
			IBean bnGroup = dbsBetGroups.fetchBean(new Equals("CODE", pGroupCode));
			
			if (bnGroup == null)
			{
				return null;
			}
			
			//group owner is already in the bet group!
			if (((Number)bnGroup.get("USER_ID")).intValue() == pUserId.intValue())
			{
				throw new SecurityException("You are already joined the bet group!");
			}
			
			//----------------------------------------------------------------------
			// Add the user to the group
			//----------------------------------------------------------------------
			
			DBStorage dbsUserBegr = new DBStorage();
			dbsUserBegr.setDBAccess(dba);
			dbsUserBegr.setWritebackTable("USER_BEGR");
			dbsUserBegr.open();
			
			//check if user already joined the group
			IBean bnUserBegr = dbsUserBegr.fetchBean(new Equals("BEGR_ID", bnGroup.get("ID")).and(new Equals("USER_ID", pUserId)));
			
			if (bnUserBegr != null)
			{
				throw new SecurityException("You are already joined the bet group!");
			}
			
			bnUserBegr = new Bean();
			bnUserBegr.put("USER_ID", pUserId);
			bnUserBegr.put("BEGR_ID", bnGroup.get("ID"));
			
			bnUserBegr = dbsUserBegr.insertBean(bnUserBegr);
			
			dba.commit();
			
			return (String)bnGroup.get("BETGROUP");
		}
		catch (Throwable th)
		{
			if (dba != null)
			{
				try
				{
					dba.rollback();
				}
				catch (Exception e)
				{
					//nothing to be done
				}
			}
			
			throw new RuntimeException("Joining group failed!", th);
		}
	}
	
	/**
	 * Gets the application configuration from the database.
	 * 
	 * @return the application configuration as key/value hashtable
	 * @throws Throwable if the database access causes an exception
	 */
	private Hashtable<Object, Object> getConfig() throws Throwable
	{
		DBStorage dbsConfig = (DBStorage)SessionContext.getCurrentSession().get("config");
		
		List<IBean> liBeans = dbsConfig.fetchBean(null, null, 0, -1);
		
		if (liBeans != null)
		{
			Hashtable<Object, Object> htConfig = BeanUtil.toHashtable(liBeans, "PARAMETER", "VALUE");

			return htConfig;
		}
		else
		{
			return new Hashtable<Object, Object>();
		}
	}

	/**
	 * Creates a "random" string.
	 * 
	 * @param pAllowedChars the allowed code characters
	 * @return a string with characters from the list of allowed characters
	 */
	String createCode(String pAllowedChars)
	{
		SecureRandom rand = new SecureRandom();
		
		StringBuilder sbPassword = new StringBuilder();
		
		int iLength = 6 + (int)(rand.nextFloat() * 5);
		
		int iChars = pAllowedChars.length();
		
		
		for (int i = 0; i < iLength; i++)
		{
			sbPassword.append(pAllowedChars.charAt((int)(rand.nextFloat() * (iChars - 1))));
		}
		
		sLastCode = sbPassword.toString();
		
		return sbPassword.toString();
	}
	
	/**
	 * Returns the last created activation key. Used for internal tests only.
	 * 
	 * @return the activation key
	 */
	String getLastActivationKey()
	{
		return sLastActivationKey;
	}
	
	/**
	 * Returns the last created code. Used for internal tests only.
	 * 
	 * @return the code
	 */
	String getLastCode()
	{
		return sLastCode;
	}
	
}	// UserRegistration
