/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 08.10.2009 - [JR] - creation
 * 13.10.2009 - [JR] - activationkey support
 *                   - closed statements
 * 28.10.2009 - [JR] - send activated as string because only strings will be sent to the client!                   
 */
package com.sibvisions.apps.packung.security;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.rad.remote.IConnectionConstants;
import javax.rad.server.IConfiguration;
import javax.rad.server.ISession;

import com.sibvisions.rad.server.AbstractSession;
import com.sibvisions.rad.server.config.DBObjects;
import com.sibvisions.rad.server.security.DBSecurityManager;
import com.sibvisions.util.log.ILogger;
import com.sibvisions.util.log.LoggerFactory;

/**
 * The <code>AnonymousDBSecurityManager</code> extends the {@link DBSecurityManager} and
 * supports login without user/password based on an anonymous user which will be configured
 * in the database.
 * 
 * @author Ren Jahn
 */
public class AnonymousDBSecurityManager extends DBSecurityManager
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** the name of the user_role table. */
	protected static final String TABLE_USER_ROLE 	= "USER_ROLE";
	
	/** the name fo the roles table. */
	protected static final String TABLE_ROLES 		= "ROLES";
	
	
	/** the logger. */
	private ILogger log 	= LoggerFactory.getInstance(getClass());
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Overwritten methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * {@inheritDoc}
	 */
	public synchronized void validateAuthentication(ISession pSession) throws Exception
	{
		String sActivationKey = (String)pSession.getProperty(IConnectionConstants.PREFIX_CLIENT + "activationkey");
		
		if (sActivationKey != null)
		{
			PreparedStatement ps = null;
			
			ResultSet res = null;

			Connection con = null;
			
			try
			{
				con = openConnection(pSession);
				
				IConfiguration cfgSession = pSession.getConfig();
				
				String sUserTable = DBObjects.getTableName(cfgSession, TABLE_USERS);
				String sActiveCol = DBObjects.getColumnName(cfgSession, TABLE_USERS, "ACTIVE");
				String sActKeyCol = DBObjects.getColumnName(cfgSession, TABLE_USERS, "ACTIVATIONKEY");
				
				ps = con.prepareStatement("select * from " + sUserTable +
						                  " where " + sActKeyCol + " = ?" +
						                    " and " + sActiveCol + " = 'N'");
	
				ps.setString(1, sActivationKey);
				res = ps.executeQuery();
				
				//activation key must exist
				if (res.next())
				{
					String sUserName = res.getString(DBObjects.getColumnName(cfgSession, TABLE_USERS, "USERNAME"));

					BigDecimal bdUserId = res.getBigDecimal(DBObjects.getColumnName(cfgSession, TABLE_USERS, "ID"));
					
					long lTime = res.getTimestamp(DBObjects.getColumnName(cfgSession, TABLE_USERS, "CREATED_ON")).getTime();

					//activation key must be unique!
					if (res.next())
					{
						throw new SecurityException("Invalid activation key!");
					}

					res.close();
					ps.close();

					res = null;
					
					//activation key is valid for 24 hours
					if (lTime + 86400000 < System.currentTimeMillis())
					{
						//remove the activation key and leave the user inactive
						ps = con.prepareStatement("update " + sUserTable +
		    	                                    " set " + sActKeyCol + " = null" +
			                                      " where " + sActKeyCol + " = ?");
						
						ps.setString(1, sActivationKey);
						ps.execute();
						
						ps.close();
						
						con.commit();
						
						throw new SecurityException("Activation key expired!");
					}
					
					//----------------------------------------------------------------------
					// Assign the USER role to the activated user!
					//----------------------------------------------------------------------

					ps = con.prepareStatement("select " + DBObjects.getColumnName(cfgSession, TABLE_ROLES, "ID") +
							                   " from " + DBObjects.getTableName(cfgSession, TABLE_ROLES) +
							                  " where " + DBObjects.getColumnName(cfgSession, TABLE_ROLES, "NAME") + " = ?");
					ps.setString(1, cfgSession.getProperty("/application/userrole"));
					res = ps.executeQuery();
					
					if (!res.next())
					{
						throw new SecurityException("Userrole was not found!");
					}
					else
					{
						BigDecimal bdRoleId = res.getBigDecimal(1);
						
						res.close();
						res = null;
						
						ps.close();
						
						ps = con.prepareStatement("insert into " + DBObjects.getTableName(cfgSession, TABLE_USER_ROLE) +
								                  "(" + DBObjects.getColumnName(cfgSession, TABLE_USER_ROLE, "USER_ID") + ", " +
								                    DBObjects.getColumnName(cfgSession, TABLE_USER_ROLE, "ROLE_ID") + 
								                  ") VALUES (?, ?)");
						
						ps.setBigDecimal(1, bdUserId);
						ps.setBigDecimal(2, bdRoleId);
						ps.execute();
						
						ps.close();
						
						//----------------------------------------------------------------------
						//activate user
						//----------------------------------------------------------------------
						
						String sYes = DBObjects.getYesValue(cfgSession);
						
						ps = con.prepareStatement("update " + sUserTable +
							    	                " set " + sActKeyCol + " = null, " + sActiveCol + " = ?, " + 
							    	                          DBObjects.getColumnName(cfgSession, TABLE_USERS, "REGISTERED") + " = ?" +
								                  " where " + sActKeyCol + " = ?");
						
						ps.setString(1, sYes);
						ps.setString(2, sYes);
						ps.setString(3, sActivationKey);
						
						int iResult = ps.executeUpdate();
						
						//paranoid check
						if (iResult != 1)
						{
							throw new SecurityException("Activation failed!");
						}
						
						ps.close();
						
						con.commit();

						//----------------------------------------------------------------------
						// client stuff
						//----------------------------------------------------------------------
						
						((AbstractSession)pSession).setUserName(sUserName);
						
						//unset the property
						pSession.setProperty(IConnectionConstants.PREFIX_CLIENT + "activationkey", null);
	
						//important information for the client!
						pSession.setProperty(IConnectionConstants.PREFIX_CLIENT + "activated", Boolean.toString(true));
						
						//sent through Session, bot only after LCO access!
						pSession.setProperty(IConnectionConstants.PREFIX_SERVER + IConnectionConstants.PREFIX_SESSION + "REGISTERED", "Y");
					}
				}
				else
				{
					throw new SecurityException("Unknown activation key!");
				}
			}
			catch (SQLException sqle)
			{
				sqle.printStackTrace();
				
				log.debug(sqle);

				if (con != null)
				{
					try
					{
						con.rollback();
					}
					catch (SQLException ex)
					{
						//nothing to be done
					}
				}

				throw new SecurityException("User activation for application '" + pSession.getApplicationName() + "' is not possible!");
			}
			finally
			{
				if (ps != null)
				{
					try
					{
						ps.close();
					}
					catch (Exception ex)
					{
						//nothing to be done
					}
					finally
					{
						ps = null;
					}
				}
				
				if (res != null)
				{
					try
					{
						res.close();
					}
					catch (Exception ex)
					{
						//nothing to be done
					}
					finally
					{
						res = null;
					}
				}
			}
		}
		else if (Boolean.valueOf((String)pSession.getProperty(IConnectionConstants.PREFIX_CLIENT + "login.anonymous")))
		{
			PreparedStatement ps = null;
			
			ResultSet res = null;
			
			
			try
			{
				Connection con = openConnection(pSession);
				
				IConfiguration cfgSession = pSession.getConfig();
				
				ps = con.prepareStatement("select * from " + DBObjects.getTableName(cfgSession, TABLE_USERS) +
						                                    " where " + DBObjects.getColumnName(cfgSession, TABLE_USERS, "ANONYMOUS") + " = ?");
	
				ps.setString(1, DBObjects.getYesValue(cfgSession));
				res = ps.executeQuery();
				
				//use the first anonymous user!
				if (res.next())
				{
					((AbstractSession)pSession).setUserName(res.getString(DBObjects.getColumnName(cfgSession, TABLE_USERS, "USERNAME")));
				}
				else
				{
					throw new SecurityException("Anonymous login not allowed!");
				}
	
				//unset the property
				pSession.setProperty(IConnectionConstants.PREFIX_CLIENT + "login.anonymous", null);
			}
			catch (SQLException sqle)
			{
				log.debug(sqle);
				
				throw new SecurityException("Anonymous authentication for application '" + pSession.getApplicationName() + "' is not possible!");
			}
			finally
			{
				if (ps != null)
				{
					try
					{
						ps.close();
					}
					catch (Exception ex)
					{
						//nothing to be done
					}
					finally
					{
						ps = null;
					}
				}
				
				if (res != null)
				{
					try
					{
						res.close();
					}
					catch (Exception ex)
					{
						//nothing to be done
					}
					finally
					{
						res = null;
					}
				}
			}
		}
		else
		{
			super.validateAuthentication(pSession);
		}
	}
	
}	//AnonymousDBSecurityManager
