/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 25.10.2009 - [JR] - creation
 * 26.10.2009 - [JR] - testComplete based on ant script
 */
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.sql.Connection;
import java.sql.DriverManager;

import junit.framework.Assert;

import org.dbunit.database.DatabaseConfig;
import org.dbunit.database.DatabaseConnection;
import org.dbunit.database.IDatabaseConnection;
import org.dbunit.dataset.CachedDataSet;
import org.dbunit.dataset.FilteredDataSet;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.filter.ITableFilter;
import org.dbunit.dataset.filter.SequenceTableFilter;
import org.dbunit.dataset.xml.FlatDtdDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSet;
import org.dbunit.dataset.xml.FlatXmlProducer;
import org.dbunit.dataset.xml.FlatXmlWriter;
import org.dbunit.ext.oracle.Oracle10DataTypeFactory;
import org.dbunit.operation.DatabaseOperation;
import org.junit.Test;
import org.xml.sax.InputSource;

import com.sibvisions.util.type.FileUtil;

/**
 * Tests the export and import of packung table data. The export will be done from oracle
 * and the import will be done for derby.
 * 
 * @author rjahn
 */
public class TestDatabase
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Creates a new oracle connection.
	 * 
	 * @return the established connection
	 * @throws Exception if the connection can not be established
	 */
    private IDatabaseConnection getOracleConnection() throws Exception 
    {
		Class.forName("oracle.jdbc.OracleDriver");
		
		Connection jdbcConnection = DriverManager.getConnection("jdbc:oracle:thin:@sibserver:1521:XE", "packung_clean", "packung");
		
		return new DatabaseConnection(jdbcConnection);
    }

	/**
	 * Creates a new derby connection.
	 * 
	 * @return the established connection
	 * @throws Exception if the connection can not be established
	 */
    private IDatabaseConnection getDerbyConnection() throws Exception 
    {
		Class.forName("org.apache.derby.jdbc.ClientDriver");
		
		Connection jdbcConnection = DriverManager.getConnection("jdbc:derby://localhost/packung", "packung", "packung");
		
		return new DatabaseConnection(jdbcConnection);
    }

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Test methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Exports table data for packung from the oracle database.
	 * 
	 * @throws Exception if the export fails
	 */
	@Test
    public void testExport() throws Exception
    {
    	IDatabaseConnection connection = getOracleConnection();

    	DatabaseConfig config = connection.getConfig();

    	config.setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY, new Oracle10DataTypeFactory());
    	config.setFeature("http://www.dbunit.org/features/skipOracleRecycleBinTables", true);

    	ITableFilter filter = new SequenceTableFilter(new String[] {
    			//master data
                "LANGUAGES",
                "CONFIG",
                "USERS",
    			"AUTOLOGIN",
                "WORKSCREENS",
                "TEAMS",
                "ROLES",
                "RANKS",
                
                //referenced (ordered to solve foreign key problems)
                "INFOTEXTS",
                "ROLE_WOSC",
                "USER_ROLE",
                "COMPETITIONTYPES",
                "COMPETITIONS",
                "STAGES",
                "GROUPS",
                "BETGROUPS",
                "USER_BEGR",
                "MATCHDAYS",
                "MATCHPLACES",
                "GROUPTEAMS",
                "MATCHES",
                "COTY_TEAM",
                "TOPBETS",
                "BETS"});
    	
    	IDataSet dataset = new FilteredDataSet(filter, connection.createDataSet());
    	
    	FlatXmlDataSet.write(dataset, new FileOutputStream(new File("db/export.xml")));

    	//DTD
    	FlatDtdDataSet.write(dataset, new FileOutputStream(new File("db/export.dtd")));
    	
    	FlatXmlWriter datasetWriter = new FlatXmlWriter(new FileOutputStream(new File("db/export_dtd.xml"))); 
    	datasetWriter.setDocType("export.dtd"); 
    	datasetWriter.write(dataset);
    }
	
	/**
	 * Imports the table data into the derby database.
	 * 
	 * @throws Exception if the import fails
	 */
	@Test
	public void testImport() throws Exception
	{
		FlatXmlProducer producer = new FlatXmlProducer(new InputSource("db/export_dtd.xml"));
		producer.setValidating(true);
		IDataSet dataSet = new CachedDataSet(producer);

		dataSet = new FlatXmlDataSet(new FileInputStream(new File("db/export.xml")), true);
		
        IDatabaseConnection connection = getDerbyConnection();

    	DatabaseConfig config = connection.getConfig();
    	config.setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY, new Oracle10DataTypeFactory());
        
        DatabaseOperation.CLEAN_INSERT.execute(connection, dataSet);
	}
    
	/**
	 * Calls the db create script with derby ij tool.
	 * 
	 * @throws Exception if the script execution fails
	 */
	@Test
	public void testCreateScript() throws Exception
	{
		File fiResult = new File("db/ij_create.txt");

		System.setProperty("ij.Driver", "org.apache.derby.jdbc.ClientDriver");
		System.setProperty("ij.user", "packung");
		System.setProperty("ij.password", "packung");
		System.setProperty("ij.connection.packung.database", "jdbc:derby://localhost:1527/packung;create=true;");
		System.setProperty("ij.outfile", fiResult.getAbsolutePath());
		
	    org.apache.derby.tools.ij.main(new String[] {new File("db/create.sql").getAbsolutePath()});

	    //Check execution errors
	    String sContent = new String(FileUtil.getContent(new FileInputStream(fiResult)));
	    
	    int iErrorPos = sContent.indexOf("ERROR"); 
	    
	    if (iErrorPos >= 0)
	    {
	    	//show next 500 characters as error detail
	    	System.out.println(sContent.substring(Math.max(0, iErrorPos - 500), 
	    			                              Math.min(sContent.indexOf('\n', iErrorPos), sContent.length())));
	    	
	    	Assert.fail("Script error");
	    }
	    
		//Class.forName("org.apache.derby.jdbc.ClientDriver");
		
		//Connection con = DriverManager.getConnection("jdbc:derby://localhost/packung;create=true", "packung", "packung");
		//org.apache.derby.tools.ij.runScript(con, new FileInputStream("db/create.sql"), "cp1252", new FileOutputStream("D:\\create.txt"), "cp1252");
	}
	
	/**
	 * Tests db creation, export from oracle and import into derby db.
	 * 
	 * @throws Exception if the test fails
	 */
	@Test
	public void testComplete() throws Exception
	{
		testCreateScript();
		testExport();
		testImport();
	}
	
}	// TestDatabase
