/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 15.10.2009 - [JR] - creation
 * 17.10.2009 - [JR] - activation check
 * 21.10.2009 - [JR] - check REGISTERED flag
 * 26.10.2009 - [JR] - joinPrivateGroup test
 */
package com.sibvisions.apps.packung.business;

import java.io.File;
import java.math.BigDecimal;
import java.util.Hashtable;
import java.util.Map;

import javax.rad.application.ILauncher;
import javax.rad.remote.AbstractConnection;
import javax.rad.remote.IConnectionConstants;
import javax.rad.remote.MasterConnection;

import junit.framework.Assert;

import org.junit.BeforeClass;
import org.junit.Test;

import com.sibvisions.rad.server.DirectServerConnection;
import com.sibvisions.util.type.ResourceUtil;

/**
 * Tests the functionality of {@link UserRegistration}.
 * 
 * @author Ren Jahn
 */
public class TestUserRegistration
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Initializes the base config directory of the server.
	 */
	@BeforeClass
	public static void beforeClass()
	{
		System.setProperty("Configuration.basedir", 
				           new File(ResourceUtil.getPathForClass(ResourceUtil.getFqClassName(TestUserRegistration.class))).getParent());
	}

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Creates a new connection.
	 * 
	 * @param pConProps additional connection properties
	 * @return the connection
	 * @throws Throwable if the connection can not be opened
	 */
	private AbstractConnection createConnection(Hashtable<String, String> pConProps) throws Throwable
	{
		MasterConnection macon = new MasterConnection(new DirectServerConnection());
		macon.setApplicationName("packung");
		macon.setUserName("packung");
		macon.setPassword("packung");
		
		macon.setProperty(IConnectionConstants.PREFIX_CLIENT + ILauncher.PARAM_SERVERBASE, "http://localhost/packung");
		
		if (pConProps != null)
		{
			for (Map.Entry<String, String> entry : pConProps.entrySet())
			{
				macon.setProperty(entry.getKey(), entry.getValue());
			}
		}
		
		macon.open();
		
		return macon;
	}

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Test methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Tests the {@link UserRegistration#delete(String)} method.
	 * 
	 * @throws Throwable if the test fails
	 */
	@Test
	public void testDelete() throws Throwable
	{
		AbstractConnection con = createConnection(null);
		
		try
		{
			con.call("registration", "delete", "unknownuser");
			
			Assert.fail("User 'unknownuser' found!");
		}
		catch (SecurityException se)
		{
			Assert.assertEquals("User 'unknownuser' was not found!", se.getMessage());
		}
	
		con.close();
	}
	
	/**
	 * Tests the {@link UserRegistration#create(String, String, String, String, java.math.BigDecimal)} method.
	 * This method requires the ID 40 in the languages table, otherwise the test fails!
	 * 
	 * @throws Throwable if the test fails
	 */
	@SuppressWarnings("unchecked")
	@Test
	public void testCreate() throws Throwable
	{
		AbstractConnection con = createConnection(null);

		try
		{
			con.call("registration", "delete", "testuser");
		}
		catch (SecurityException se)
		{
			//expected
		}
		
		Hashtable<String, String> htResult = (Hashtable)con.call("registration", "create", 
				                                                 "Ren", "Jahn", 
				                                                 "testuser", "rjahn@mysibvisions.com", 
				                                                 "testpwd", "testpwd", null);
		
		Assert.assertEquals("Registration failed with form error!", "{}", htResult.toString());

		try
		{
			con.call("registration", "delete", "testuser");
		}
		catch (SecurityException se)
		{
			//expected
		}

		htResult = (Hashtable)con.call("registration", "create", 
				                       "Ren", "Jahn", "testuser", "rjahn@mysibvisions.com", 
				                       "testpwd", "testpwd", BigDecimal.valueOf(40));
		
		Assert.assertEquals("Registration failed with form error!", "{}", htResult.toString());
		
		//Finally check if it's possible to activate the user
		
		//Possible with DirectServerConnection!
		UserRegistration reg = (UserRegistration)con.callAction("getRegistration");

		String sKey = reg.getLastActivationKey();
		
		con.close();

		Hashtable<String, String> htProps = new Hashtable<String, String>();
		htProps.put(IConnectionConstants.PREFIX_CLIENT + "activationkey", sKey);
		
		con = createConnection(htProps);

		Assert.assertNotNull("Missing registered property!", 
                             con.getProperty(IConnectionConstants.PREFIX_SERVER + IConnectionConstants.PREFIX_SESSION + "REGISTERED"));
		
		Assert.assertNotNull("Activation failed!", con.getProperty(IConnectionConstants.PREFIX_CLIENT + "activated"));
		Assert.assertNull("Activation failed!", con.getProperty(IConnectionConstants.PREFIX_CLIENT + "activationkey"));
		
		con.close();
	}

	/**
	 * Tests the {@link UserRegistration#createCode(String)} method.
	 */
	@Test
	public void testCreateCode()
	{
		UserRegistration reg = new UserRegistration();
		
		Assert.assertNotNull(reg.createCode("ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789abcdefghijklmnopqrstuvwxyz"));
	}

	/**
	 * Tests the {@link UserRegistration#setNewPassword(String, String)} method.
	 * 
	 * @throws Throwable if the test fails
	 */
	@Test
	public void testSetNewPassword() throws Throwable
	{
		AbstractConnection con = createConnection(null);
		
		testCreate();
		
		con.call("registration", "setNewPassword", "testuser", null);
		
		//Possible with DirectServerConnection!
		UserRegistration reg = (UserRegistration)con.callAction("getRegistration");

		String sPwd = reg.getLastCode();
		
		con.close();
		
		con.setUserName("testuser");
		con.setPassword(sPwd);
		con.open();
		
		((MasterConnection)con).setNewPassword(sPwd, "testpwd");
		
		testDelete();
	}

	/**
	 * Tests the {@link UserRegistration#sendPrivateGroup(BigDecimal, String, String)} method.
	 * 
	 * @throws Throwable if the test fails
	 */
	@Test
	public void testSendPrivateGroup() throws Throwable
	{
		AbstractConnection con = createConnection(null);
		
		con.call("registration", "sendPrivateGroup", new BigDecimal(0), "de", "rjahn@mysibvisions.com");
		
		//Possible with DirectServerConnection!
		UserRegistration reg = (UserRegistration)con.callAction("getRegistration");

		String sCode = reg.getLastCode();
		
		Assert.assertNotNull("Group code not created!", sCode);
		
		con.close();
	}
	
	/**
	 * Tests the {@link UserRegistration#joinPrivateGroup(BigDecimal, String)} method.
	 * 
	 * @throws Throwable if the test fails
	 */
	@Test
	public void testJoinPrivateGroup() throws Throwable
	{
		AbstractConnection con = createConnection(null);
		
		//be sure theat the user wit ID 14 exists and is not a member of the bet group identified by the code!
		con.call("registration", "joinPrivateGroup", new BigDecimal(14), "0P2S6UA3R");
	}
	
}	// TestUserRegistration
